function label = BJwknKmeans(U, w, init)
% Perform weighted kernel kmeans clustering.
% Input:
%   K: n x n data matrix
%   init: either number of clusters (k) or initial label (1xn)
% Output:
%   label: 1 x n sample labels
%   model: trained model structure
%   energy: optimization target value
% Reference: Kernel Methods for Pattern Analysis
% by John Shawe-Taylor, Nello Cristianini
% Written by Mo Chen (sth4nth@gmail.com).
% 
% Modified for DisKmeans by Wen Huang
% 
n = size(U, 1);

WinvsqrtU = spdiags(w.^(-0.5), 0, n, n) * U;

KD = zeros(1, n);
for i = 1 : n
    KD(i) = WinvsqrtU(i, :) * WinvsqrtU(i, :)';
end

% n = size(K,2);
if numel(init)==1
    k = init;
    label = ceil(k*rand(1,n));
elseif numel(init)==n
    label = init;
    k = max(init);
end

last = zeros(1,n);
while any(label ~= last)
    [~,~,last(:)] = unique(label);   % remove empty clusters
    E = sparse(last,1:n,w);
    E = E./sum(E,2);
    T = (E * WinvsqrtU) * WinvsqrtU';
    [val, label] = min( repmat(KD, size(T, 1), 1) - 2 * T + dot(T,E,2) , [], 1);
%     [val, label] = max(T-dot(T,E,2)/2,[],1);
end
label = label';