  function [xopt, fs, Ds, sparsity, comtime, avar] = driver(A, SallA, x0, n, p, tol, maxiter, mu, innertol, Ftol)
% min_x -trace(x' A x) + mu \|x\|_1

    % parameters for line search
    delta = 0.0001;
    gamma = 0.5;
    [n,~]=size(A);
    
    % functions for the optimization problem
    fhandle = @(x)f(x, A, mu);
    gfhandle = @(x)gf(x, A, mu);
    fprox = @prox;
    fcalJ = @calJ;
    
    % functions for the manifold
%     fcalA = @calA;
%     fcalAstar = @calAstar;

    fcalA = @embedded_intr_c; % modified
    fcalAstar = @embedded_full_c; % modified
    
    xinitial.main = x0;
    tmp = A * sparse(ones(n, 1)); 
    SallA = sum(sum(A));
    %     B = A - tmp * tmp' /SallA; 
    L = 2 * max(eigs(A-tmp * tmp' /SallA)); %L=2*norm(B), where tmp = A * ones(n, 1); B = A - tmp * tmp' / (ones(1, n) * tmp); 
    
    tic
    [xopt, fs, Ds] = solver(A, fhandle, gfhandle, fcalA, fcalAstar, fprox, fcalJ, SallA, xinitial, L, tol, delta, gamma, maxiter, mu, innertol, Ftol);
    comtime = toc;
    fprintf('comtime:%1.6f\n',comtime);

    sparsity = sum(sum(abs(xopt.main) < 1e-5)) / (n * p);
    xopt = xopt.main;
%     fprintf('sparsity:%1.3f\n',sum(sum(abs(xopt) < 1e-5)) / (n * p));
    
end

function [xopt, fs, Ds] = solver(A,fhandle, gfhandle, fcalA, fcalAstar, fprox, fcalJ, SallA, x0, L, tol, delta, gamma, maxiter, mu, innertol, Ftol)
%     global Z
    err = inf;
    nf=0;
    ng=0;
    nR=0;
    [fx1, x1] = fhandle(x0);
    nf=nf+1;
    gfx1 = gfhandle(x1);
    ng=ng+1;
    t = 1 / L;
    t0 = t;
    iter = 0;
    totalinneriterx=0;
    totalinneritery=0;
    totaltotalCGiter=0;
    nrestart=0;% number of restart(number of safeguard takes effect)
    fs(iter + 1) = fx1;
    [n, p] = size(x0.main);
    zeta = zeros(n, p); % this is R_{x_k}^{-1}(y_k)
    s1 = 1;
%     Dyinitial = zeros(p, p);
%     Dxinitial = zeros(p, p);

%%%% modify the initial for Lambda in fcalAtar%%%%%
    Dyinitial = zeros(0.5*p*(p+1)+n-p, 1);
    Dxinitial = zeros(0.5*p*(p+1)+n-p, 1);
    
    totalbt = 0;
%     innertol = max(1e-13, min(1e-11,1e-3*sqrt(tol)*t^2));
    x1 = x0;
    y1 = x0;
    gfy1 = gfx1;
    fy1 = fx1;
    fx0 = fx1;
    gfx0 = gfx1;
    while(err > tol && fx1 > Ftol + 1e-7 && iter < maxiter)
        
        % safeguard
        if(mod(iter, 5) == 0)
            [Dx, Dxinitial, inneriterx, totalCGiterx] = finddir(iter, x0, gfx0, t, fcalA, fcalAstar, fprox, fcalJ, mu, Dxinitial, innertol);
            
            if(iter == 0)
                err0 = (norm(Dx, 'fro') / t);
            end
            
            totalinneriterx=totalinneriterx+inneriterx; %%%%%%totalinneriterx
            totaltotalCGiter=totaltotalCGiter+totalCGiterx;
            alpha = 1;
            xc = R(x0, alpha * Dx);
            nR=nR+1;
            [fxc, xc] = fhandle(xc);
            nf=nf+1;
            btiter = 0;
            while(fxc > fx0 - delta * alpha * norm(Dx, 'fro')^2 && btiter < 20)
                alpha = alpha * gamma;
                xc = R(x0, alpha * Dx);
                nR=nR+1;
                [fxc, xc] = fhandle(xc);
                nf=nf+1;
                btiter = btiter + 1;
                totalbt = totalbt + 1;
            end
            if(btiter == 20)
%                 if(innertol == 1e-20) %update
                    fprintf('line search fails!\n');
                    break;
%                 end
%                 innertol = max(innertol * 1e-2, 1e-20);
                
%                 continue;
            end
%             fx0 - fxc
            if(iter ~= 0 && fxc < fx2) % if safeguard takes effect
                gfxc = gfhandle(xc);
                ng=ng+1;
                y1 = xc;gfy1 = gfxc; fy1 = fxc;
                x1 = xc;
                s1 = 1;
                nrestart=nrestart+1;
            end
            % update x0
            x0 = x1;
            fx0 = fhandle(x0);%update
            nf=nf+1;
            gfx0 = gfhandle(x0);
            ng=ng+1;
        end
        [Dy, Dyinitial, inneritery, totalCGitery] = finddir(iter, y1, gfy1, t, fcalA, fcalAstar, fprox, fcalJ, mu, Dyinitial, innertol);
        totalinneritery=totalinneritery+inneritery; %%%%%%totalinneritery
        totaltotalCGiter=totaltotalCGiter+totalCGitery;
        alpha = 1;
        x2 = R(y1, alpha * Dy);
        nR=nR+1;

        [fx2, x2] = fhandle(x2);
        nf=nf+1;
        
        s2 = (1 + sqrt(1 + 4 * s1 * s1)) / 2;
        zeta = (- (s1 - 1) / s2) * proj1(x1.main - x2.main,x2); %modification of Rinv
        
        iter = iter + 1;
        fs(iter + 1) = fx2;
        err = (norm(Dx, 'fro') / t) / err0;
        Ds(iter) = norm(Dx, 'fro');

        if(mod(iter, 10) == 0)
            fprintf('iter:%d, f:%e, innertol:%e, err:%e, inneriterx:%d, inneritery:%d, btiter:%d, inneriterx:%d, inneritery:%d, CGiter:%d \n', iter, fx2, innertol, err, inneriterx, inneritery, btiter, totalinneriterx, totalinneritery, totaltotalCGiter);
        end
        
        y2 = R(x2, zeta);
        nR=nR+1;
        [fy2, y2] = fhandle(y2);
        nf=nf+1;
        gfy2 = gfhandle(y2);
        ng=ng+1;
        fy1 = fy2; gfy1 = gfy2;
        y1 = y2;
        s1 = s2;
        x1 = x2; fx1 = fx2;
    end
    
    fprintf('iter:%d,totalinneriterx:%d, totalinneritery:%d, totaltotalCGiter:%d, nrestart:%d, f:%.16f, err:%e,totalbt:%d, nf:%d, ng:%d, nR:%d\n', iter,totalinneriterx, totalinneritery, totaltotalCGiter, nrestart, fx2, err, totalbt, nf, ng, nR);
    xopt = x2;
    

end

function output = proj(x, eta) %Stiefel to Tangent space of Stiefel
    tmp = x.main' * eta;
    output = eta - x.main * (tmp + tmp')/2;
end

function output = R(x, eta)
    [n, p] = size(x.main);
    [Q,R] = qr(x.main + eta,0);
    [U,S,V] = svd(R);
    y = Q*(U*V');
    q_star = y' * ones(n, 1);
    q_star = q_star / norm(q_star);
    
    output.main = (ones(n, 1) / sqrt(n)) * q_star' + y - (y * q_star) * q_star';
    
end

%%------Need to modify Rinv------%%
function output = Rinv(x, y)
    A = x.main' * y.main;
    S = lyap(A, -2 * eye(size(A)));
    output = y.main * S - x.main;
end

%%%projection from T_X(St(p,n)) to T_X(calF)
function output=proj1(eta, x)
    [n, p] = size(x.main);
    tmp1 = x.main' * eta;
    alpha = x.main' * ones(n, 1);
    alpha = alpha / norm(alpha);
    tmp2 = eta - (eta * alpha) * alpha';
    output = x.main * ((tmp1 - tmp1') / 2) + tmp2 - x.main * (x.main' * tmp2);
end

function [output, x] = f(x, A, mu)
    [n,~]=size(A);
    temp=A*ones(n, 1); 
    SallA = sum(sum(A));
    Z=sparse(x.main'*temp);
    x.Ax = A * x.main-temp*Z'/SallA;
    %tmp = norm(x.Ax, 'fro');
    output = - sum(x.main(:) .* x.Ax(:)) + mu * sum(abs(x.main(:)));
end

function output = gf(x, A, mu)
   
    if(~isfield(x, 'Ax'))
        [n,~]=size(A);
        temp=A*sparse(ones(n, 1)); 
        SallA = sum(sum(A));
        Z=sparse(x.main'*temp);
        x.Ax = A * x.main-temp*Z'/SallA;
    end
    gfx = -2 * (x.Ax);
    tmp = gfx' * x.main;
    output = gfx - x.main * ((tmp + tmp') / 2); 
end

% compute E(Lambda)
function ELambda = E(Lambda, BLambda, x, gfx, t, fcalA, fcalAstar, fprox, fcalJ, mmu)% x is a struct
    if(length(BLambda) == 0)
        BLambda = x.main - t * (gfx - fcalAstar(Lambda, x));
    end
    DLambda = fprox(BLambda, t, mmu) - x.main;
    ELambda = fcalA(DLambda, x);
end

% compute calG(Lambda)[d]
function GLambdad = GLd(Lambda, d, BLambda, Blocks, x, gfx, t, fcalA, fcalAstar, fprox, fcalJ, mmu) %x is a sturct
        GLambdad = t * fcalA(fcalJ(BLambda, fcalAstar(d, x), t, mmu), x); %not struct
end

% Use semi-Newton to solve the subproblem and find the search direction
function [output, Lambda, inneriter, totalCGiter] = finddir(iter, xx, gfx, t, fcalA, fcalAstar, fprox, fcalJ, mmu, x0, innertol)
%     x = xx.main;
    x = Get_info_intr(xx);
%     x=xx;
    lambda = 0.2;
    nu = 0.9999;
    tau = 0.1;
    eta1 = 0.2; eta2 = 0.75;
    gamma1 = 2; gamma2 = 5;
    alpha = 0.1;
    beta = 1 / alpha / 100;
%     [n, p] = size(x);
    [n, p] = size(x.main);

    z = x0;
    n2z=norm(z);
    fcalAstarz = fcalAstar(z,x);
    n2zstar=norm(fcalAstarz);
    BLambda = x.main - t * (gfx - fcalAstarz); % output of fcalAstar is a n*p matrix; x is a struct
    Fz = E(z, BLambda, x, gfx, t, fcalA, fcalAstar, fprox, fcalJ, mmu);
    
    nFz = norm(Fz, 'fro');
    nnls = 5;
    xi = zeros(nnls, 1);% for non-monotonic linesearch
    xi(nnls) = nFz;
    maxiter = 10;
    times = 0;
    Blocks = cell(p, 1);
    totalCGiter=0;
    while(nFz * nFz > innertol && times < maxiter) % while not converge, find d and update z
        mu = lambda * max(min(nFz, 0.1), 1e-11);
        Axhandle = @(d)GLd(z, d, BLambda, Blocks, x, gfx, t, fcalA, fcalAstar, fprox, fcalJ, mmu) + mu * d;
        [d, CGiter, nr] = myCG(Axhandle, -Fz, tau, lambda * nFz, 5); % update d
%         fprintf('CGiter:%d\n', CGiter);
        totalCGiter=totalCGiter+CGiter;
        u = z + d;
        Fu = E(u, [], x, gfx, t, fcalA, fcalAstar, fprox, fcalJ, mmu); 
        nFu = norm(Fu, 'fro');

        if(nFu < nu * max(xi))
            z = u;
            Fz = Fu;
            nFz = nFu;
            xi(mod(times, nnls) + 1) = nFz;
            status = 'success';
        else
            rho = - sum(Fu(:) .* d(:)) / norm(d, 'fro')^2;
            if(rho >= eta1)
                v = z - sum(sum(Fu .* (z - u))) / nFu^2 * Fu;
                Fv = E(v, [], x, gfx, t, fcalA, fcalAstar, fprox, fcalJ, mmu);
                nFv = norm(Fv, 'fro');
                if(nFv <= nFz)
                    z = v;
                    Fz = Fv;
                    nFz = nFv;
                    status = 'safegard success projection';
                else
                    z = z - beta * Fz;
                    Fz = E(z, [], x, gfx, t, fcalA, fcalAstar, fprox, fcalJ, mmu);
                    nFz = norm(Fz, 'fro');
                    status = 'safegard success fixed-point';
                end
            else
%                 fprintf('unsuccessful step\n');
                status = 'safegard unsuccess';
            end
            if(rho >= eta2)
                lambda = max(lambda / 4, 1e-5);
            elseif(rho >= eta1)
                lambda = (1 + gamma1) / 2 * lambda;
            else
                lambda = (gamma1 + gamma2) / 2 * lambda;
            end
        end
        BLambda = x.main - t * (gfx - fcalAstar(z, x));%
%         fprintf(['iter:%d, nFz:%e, lambda:%e, mu:%e, nFu:%e, xi:%e, ' status '\n'], times, nFz, lambda, mu, nFu, max(xi));
        times = times + 1;
    end
%     times
%     if(times == maxiter)
%         fprintf('SSNiter max\n');
%     end
%     fprintf('totalCGiter:%d\n', totalCGiter);
    Lambda = z;
    inneriter = times;
    tmp = fprox(BLambda, t, mmu) - x.main;%
    output = proj1(tmp, x);
end

function output = prox(X, t, mu) %output is the same type of X, not struct
    output = min(0, X + t * mu) + max(0, X - t * mu);
end

function output = calA(Z, U) % U \in St(p, n)
    tmp = Z' * U;
    output = tmp + tmp';
end

function output = calAstar(Lambda, U) % U \in St(p, n)
    output = U * (Lambda + Lambda');
end

function output = calJ(y, eta, t, mu)
    output = (abs(y) > mu * t) .* eta;
end

function [output, k, nr] = myCG(Axhandle, b, tau, lambdanFz, maxiter)
    x = zeros(size(b));
    r = b;% - Axhandle(x);
    p = r;
    k = 0;
    while(norm(r, 'fro') > tau * min(lambdanFz * norm(x, 'fro'), 1) && k < maxiter)
        Ap = Axhandle(p);
        alpha = r(:)' * r(:) / (p(:)' * Ap(:));
        x = x + alpha * p;
        rr0 = r(:)' * r(:);
        r = r - alpha * Ap;
        beta = r(:)' * r(:) / rr0;
        p = r + beta * p;
        k = k + 1;
    end
    nr = norm(r, 'fro');
    output = x;
end

function x = Get_info_intr(x)
    [x.HHR, x.taus] = QRmatrices(x.main);
end

function output = embedded_intr_c(eta, x) % output is a vector
%     if(~isfield(x, 'HHR'))
%         x = Get_info_intr(x);
%     end
    [n, p] = size(x.main);
    OmegaK = HHRTeta(x.main, eta, x.HHR, x.taus);
    omega = OmegaK(1:p, :);
    omega = (omega + omega') / 2; % symmetric
    K = OmegaK(p+1:end, :);
    d = K * (x.main' * (ones(n,1) / (sqrt(n)))); % Get d
    r2 = sqrt(2);
    omega = r2 * omega;
    omega(1:1+size(omega,1):end) = omega(1:1+size(omega,1):end) / r2; %diagonal element 
    output = zeros(0.5 * p * (p + 1) + n - p, 1); % dim of Normal space
    indx = find(tril(ones(size(omega)))); % lower triangular part; omega is symmetric
    output(1 : 0.5 * p * (p + 1)) = omega(indx);
    output(0.5 * p * (p + 1) + 1 : end) = d;
end

function output = embedded_full_c(v, x) % output is a n*p matrix
%     if(~isfield(x, 'HHR'))
%         x = Get_info_intr(x);
%     end
    [n, p] = size(x.main);
    r2 = sqrt(2);
    omega = tril(ones(p, p));
    indx = find(omega);
    omega(indx) = v(1 : 0.5 * p * (p + 1)) / r2;
    omega = omega + omega'; 
    omega(1:1+size(omega,1):end) = omega(1:1+size(omega,1):end) * r2 / 2; %diagonal element

    d = v(0.5 * p * (p + 1) + 1 : end); 
    K = d * ((ones(n,1)' / (sqrt(n))) * x.main); % K=d*1_n^T*X
    output = HHReta(x.main, [omega; K], x.HHR, x.taus);
end

function [HHR, taus] = QRmatrices(U)
    [n, k] = size(U);
    HHR = zeros(n, k);
    taus = zeros(k, 1);
    A = U;
    
    for i = 1 : k
        HHR(i : n, i) = A(i : n, i);
        HHR(i, i) = HHR(i, i) + norm(A(i : n, i)); %sign(A(i, i)) * 
        taus(i) = 2 / (HHR(i : n, i)' * HHR(i : n, i));
        A = A - taus(i) * HHR(:, i) * (HHR(:, i)' * A);
    end
end

function output = HHRTeta(U, eta, HHR, taus)
    [n, k] = size(U);
    output = eta;
    for i = 1 : k
        output = output - taus(i) * HHR(:, i) * (HHR(:, i)' * output);
    end
end

function output = HHReta(U, eta, HHR, taus)
    [n, k] = size(U);
    output = eta;
    for i = k : -1 : 1
        output = output - taus(i) * HHR(:, i) * (HHR(:, i)' * output);
    end
end

