## Description
This program is an implementation of the algorithm described in the paper "A Riemannian Optimization Approach to Clustering Problems", written by Wen Huang, Meng Wei, Kyle Gallivan and Paul Van Dooren. In particular, this program is to implement and test the AMANPG algorithm on two clustering problems. One clustering problem is communitiy detection problem and the other one is normalized cut problem. There are two versions of the AManPG algorithm, that is E-AManPG (exact-AManPG) and I-AManPG (Inexact-AManPG). The AManPG algorithm was implemented in Matlab.
Each feedback is very welcome. If you have found a bug or have problems, or want to give advises, please contact us:

meng.wei02@gmail.com and/or wen.huang@xmu.edu.cn  

## The structures of the codes:
- There are two folders based on different numerial applications. One folder is IAManPG_CD for applying IAManPG algorithm to comminity detection problem and the other one is IAManPG_NC for applying IAManPG algorithm to normalized cut problem.
- IAManPG_CD: 
	- Datasets: LFR benchmark networks. They are listed as "network_*.dat" files and the groud-truth communities are listed as "community_*.dat" files. LFR benchmark networks can be generated by the scripts in folder "LFR-Benchmark_UndirWeightOvp-master". There is a README.md file for instructions. For more details, please see https://www.santofortunato.net/resources.
	- The main driver codes for AManPG algorithm are in "driver_AManPG_SparseComputation.m" and "driver_AManPG_Approx_SparseComputation.m" Matlab files. The former file is the driver for E-AManPG algorithm and the latter file is the driver for I-AManPG algorithm.
	- There are three tests for this application. 
		- One is to compare the performance of E-AManPG and I-AManPG on LFR benchmark networks. It is implemented in "test_IAManPG_EAManPG_LFR_n_500_1000_5000_10000.m" file.
		- The second one is to compare the performance of I-AManPG with the state-of-the-art community detection methods (Danon, Louvain, and Spectral Newman algorithm). It is implemented in "test_IAManPG_Existing_LFR_n_1000_p_20_force_p.m". The scripts of the state-of-the-art community detection methods are in "danon_force_p.m", "cluster_jl_force_p.m", and "newman_eigenvector_method_modified_force_p_sparse_computation.m".
		- The third test is for testing the effectiveness of I-AManPG for the input number of clusters not equal to the groud-truth number of clusters. The corresponding codes are in "testGraphLFR_nested.m"
	- The remaining files in folder "IAManPG_CD" are the supportive files for community detection algorithms.
- IAmanPG_NC:
	- Datasets: There are 12 test images, for example, baby, cameraman, coins etc.. The images are in the corresponding eps files, for example, "baby.eps" etc.
	- The main driver codes for AManPG algorithm are in "driver_AManPG_SparseComputation.m" and "driver_AManPG_Approx_SparseComputation.m" Matlab files. The former file is the driver for E-AManPG algorithm and the latter file is the driver for I-AManPG algorithm.
	- We initialize the weighted kernel k-means algorithm by I- AManPG and compared with 3 other initialization methods, Bach and Jordan (BJ), Shi and Malik (SM), and METIS (ME). The comparison test is implemented in "test_AManPG_Exact_Inexact.m" file.
	- The remaining files in folder "IAManPG_NC" are the supportive files for normalized cut algorithms.

## How to run the codes:
- IAManPG_CD: There are three tests for this application. 
		- To compare the performance of E-AManPG and I-AManPG on LFR benchmark networks, we need to run "test_IAManPG_EAManPG_LFR_n_500_1000_5000_10000.m" file.
		- To compare the performance of I-AManPG with the state-of-the-art community detection methods, we need to run "test_IAManPG_Existing_LFR_n_1000_p_20_force_p.m". 
		- To test the effectiveness of I-AManPG for the input number of clusters not equal to the groud-truth number of clusters, we need to run "testGraphLFR_nested.m"
- IAmanPG_NC: To compare the performance of I-AManPG with the state-of-the-art initialization methods for normalized cut problem, we need to run "test_AManPG_Exact_Inexact.m" file.


By Meng Wei

